#!/usr/bin/env python

"""
Handlers for a person for whom scheduling is performed, inspecting outgoing
messages to obtain scheduling done externally.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.data import uri_dict, uri_item, uri_values
from imiptools.handlers import Handler
from imiptools.handlers.common import Outgoing

class PersonHandler(Handler, Outgoing):

    "Handling mechanisms specific to people."

    def set_identity(self, from_organiser=True):

        """
        Set the current user for the current object. It is usually set when
        initialising the handler, using the recipient details, but outgoing
        messages do not reference the recipient in this way.
        """

        self.user, attr = uri_item(self.obj.get_item(from_organiser and "ORGANIZER" or "ATTENDEE"))

    def _record(self, from_organiser=True):

        """
        Record details from the current object given a message originating
        from an organiser if 'from_organiser' is set to a true value.
        """

        self.set_identity(from_organiser)

        # Check for event using UID.

        if not self.have_new_object():
            return False

        # Update the object.

        if from_organiser:

            # Set the complete event or an additional occurrence.

            self.store.set_event(self.user, self.uid, self.recurrenceid, self.obj.to_node())

            # Remove additional recurrences if handling a complete event.

            if not self.recurrenceid:
                self.store.remove_recurrences(self.user, self.uid)

        else:
            # Obtain valid attendees, merging their attendance with the stored
            # object.

            attendees = self.require_attendees(from_organiser)
            self.merge_attendance(attendees)

        # Remove any associated request.

        self.store.dequeue_request(self.user, self.uid, self.recurrenceid)

        # Update free/busy information.

        self.update_event_in_freebusy(from_organiser)

        return True

    def _remove(self, from_organiser=True):

        """
        Remove details from the current object given a message originating
        from an organiser if 'from_organiser' is set to a true value.
        """

        self.set_identity(from_organiser)

        # Check for event using UID.

        if not self.have_new_object():
            return False

        # Obtain any stored object, using parent object details if a newly-
        # indicated occurrence is referenced.

        obj = self.get_stored_object_version()
        old = not obj and self.get_parent_object() or obj

        if not old:
            return False

        # Only cancel the event completely if all attendees are given.

        attendees = uri_dict(old.get_value_map("ATTENDEE"))
        all_attendees = set(attendees.keys())
        given_attendees = set(uri_values(self.obj.get_values("ATTENDEE")))
        cancel_entire_event = given_attendees == all_attendees

        # Keep the event for the organiser.

        if cancel_entire_event:
            self.store.cancel_event(self.user, self.uid, self.recurrenceid)

        # Otherwise, remove the given attendees and update the event.

        elif obj:
            for attendee in given_attendees:
                if attendees.has_key(attendee):
                    del attendees[attendee]
            obj["ATTENDEE"] = attendees.items()

        # Update the stored object with sequence information.

        if obj:
            obj["SEQUENCE"] = self.obj.get_items("SEQUENCE") or []
            obj["DTSTAMP"] = self.obj.get_items("DTSTAMP") or []

        # Update free/busy information.

        if cancel_entire_event or self.user in given_attendees:
            self.remove_event_from_freebusy()

        # Set the complete event if not an additional occurrence. For any newly-
        # indicated occurrence, use the received event details.

        self.store.set_event(self.user, self.uid, self.recurrenceid, (obj or self.obj).to_node())

        # Remove any associated request.

        self.store.dequeue_request(self.user, self.uid, self.recurrenceid)

        return True

class Event(PersonHandler):

    "An event handler."

    def add(self):
        pass

    def cancel(self):
        self._remove(True)

    def counter(self):
        pass

    def declinecounter(self):
        pass

    def publish(self):
        self._record(True)

    def refresh(self):
        pass

    def reply(self):
        self._record(False)

    def request(self):
        self._record(True)

# Handler registry.

handlers = [
    ("VEVENT",      Event),
    ]

# vim: tabstop=4 expandtab shiftwidth=4
